BOOTSTRAP = b"core:bootstrap"
REQUEST_PENDING = b"server:request_pending"

WORKER_JOIN = b"worker:join"
WORKER_LEAVE = b"worker:leave"

RUN_START = b"run:start"
RUN_STEP = b"run:step"
RUN_INTERRUPT = b"run:interrupt"
RUN_FINISH = b"run:finish"
